/**
 * Taken from visionmedia/express/lib/utils.js (MIT license)
 * Copyright (c) 2009-2012 TJ Holowaychuk
 */
var pathRegexp = function(path, keys, sensitive, strict) {
  if (toString.call(path) == '[object RegExp]') return path;
  if (Array.isArray(path)) path = '(' + path.join('|') + ')';
  path = path
    .concat(strict ? '' : '/?')
    .replace(/\/\(/g, '(?:/')
    .replace(/(\/)?(\.)?:(\w+)(?:(\(.*?\)))?(\?)?(\*)?/g, function(_, slash, format, key, capture, optional, star){
      keys.push({ name: key, optional: !! optional });
      slash = slash || '';
      return ''
        + (optional ? '' : slash)
        + '(?:'
        + (optional ? slash : '')
        + (format || '') + (capture || (format && '([^/.]+?)' || '([^/]+?)')) + ')'
        + (optional || '')
        + (star ? '(/*)?' : '');
    })
    .replace(/([\/.])/g, '\\$1')
    .replace(/\*/g, '(.*)');
  return new RegExp('^' + path + '$', sensitive ? '' : 'i');
};
var BasicRouter, Key, Route, RouteParser, normalize,
  __slice = [].slice;

Key = (function() {
  function Key(name, optional) {
    this.name = name;
    this.optional = optional != null ? optional : false;
  }

  return Key;

})();

Route = (function() {
  function Route(regexp, keys) {
    this.regexp = regexp;
    this.keys = keys;
  }

  /*
  Algorithm taken from visionmedia/express/lib/router/route.js (MIT license)
  Copyright (c) 2009-2012 TJ Holowaychuk
  */


  Route.prototype.match = function(path) {
    var i, key, matched, params, val, _i, _ref;
    params = [];
    matched = this.regexp.exec(path);
    if (!matched) {
      return false;
    }
    for (i = _i = 1, _ref = matched.length; 1 <= _ref ? _i < _ref : _i > _ref; i = 1 <= _ref ? ++_i : --_i) {
      key = this.keys[i - 1];
      val = typeof matched[i] === 'string' ? decodeURIComponent(matched[i]) : matched[i];
      if (key) {
        params[key.name] = val;
      } else {
        params.push(val);
      }
    }
    return params;
  };

  return Route;

})();

RouteParser = (function() {
  function RouteParser(sensitive, strict) {
    this.sensitive = sensitive != null ? sensitive : false;
    this.strict = strict != null ? strict : false;
  }

  RouteParser.prototype.parse = function(routeString) {
    var arr, key, keys, regexp;
    arr = [];
    regexp = pathRegexp(routeString, arr, this.sensitive, this.strict);
    keys = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = arr.length; _i < _len; _i++) {
        key = arr[_i];
        _results.push(new Key(key.name, key.optional));
      }
      return _results;
    })();
    return new Route(regexp, keys);
  };

  return RouteParser;

})();

BasicRouter = (function() {
  function BasicRouter() {
    this.routeParser = new RouteParser;
    this.routes = [];
  }

  BasicRouter.prototype.register = function(routeString, callback) {
    var route;
    if (callback == null) {
      callback = function() {};
    }
    route = this.routeParser.parse(normalize(routeString));
    return this.routes.push([route, callback]);
  };

  BasicRouter.prototype.registerNotFoundHandler = function(callback) {
    return this.notFoundHandler = callback;
  };

  BasicRouter.prototype.route = function() {
    var args, callback, found, pair, params, route, string, _i, _len, _ref;
    string = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    found = false;
    _ref = this.routes;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      pair = _ref[_i];
      route = pair[0], callback = pair[1];
      params = route.match(string);
      if (params) {
        args.unshift(params);
        callback.apply(null, args);
        found = true;
        break;
      }
    }
    if (!found && this.notFoundHandler) {
      args.unshift(string);
      this.notFoundHandler.apply(null, args);
    }
    return found;
  };

  return BasicRouter;

})();

normalize = function(routeString) {
  var lastIndex, str;
  str = routeString.replace(/[ ]/g, '');
  if (str && str[0] === '/') {
    str = str.slice(1, str.length);
  }
  lastIndex = str.length - 1;
  if (str.lastIndex === '/') {
    str = str.slice(0, lastIndex);
  }
  return str;
};

